'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var constants = require('./constants.js');
var isomorphicWebsocket = require('./internal/isomorphic-websocket.js');
var message = require('./internal/message.js');
var request = require('./internal/request.js');
var util = require('./internal/util.js');

class ScrapboxWebsocketHandler {
  options;
  socket = null;
  room = null;
  sid = 0;
  request$ = new rxjs.Subject();
  response$ = new rxjs.Subject();

  constructor(options) {
    this.options = options;
    this.request$.pipe(rxjs.concatMap(async ({
      sid,
      data,
      projectId,
      pageId
    }) => {
      const socket = await this.getActiveSocket();
      socket.send(message.toSocketIoMessagePayload(sid, data));
      const result = await rxjs.firstValueFrom(socket.message.pipe(operators.first(message.isResponseMessageOf(sid)), operators.timeout(constants.CONFIG.responseTimeout)));
      return {
        sid,
        result
      };
    })).subscribe(this.response$);
  }

  async commit({
    projectId,
    userId,
    pageId,
    parentId,
    changes
  }) {
    await this.join({
      projectId,
      pageId
    });
    return this.send({
      projectId,
      pageId,
      data: {
        method: 'commit',
        data: {
          kind: 'page',
          userId,
          projectId,
          pageId,
          parentId,
          changes: request.createChanges(changes, userId),
          cursor: null,
          freeze: true
        }
      }
    });
  }

  async join({
    projectId,
    pageId
  }) {
    var _this$room, _this$room2;

    // join room if needed
    if (((_this$room = this.room) === null || _this$room === void 0 ? void 0 : _this$room.projectId) === projectId && ((_this$room2 = this.room) === null || _this$room2 === void 0 ? void 0 : _this$room2.pageId) === pageId) {
      return;
    }

    await this.send({
      projectId,
      pageId,
      data: {
        method: 'room:join',
        data: {
          pageId,
          projectId,
          projectUpdatesStream: false
        }
      }
    });
    this.room = {
      projectId,
      pageId
    };
  }

  async send({
    data,
    projectId,
    pageId
  }) {
    const sid = String(this.sid++);
    this.request$.next({
      sid,
      data,
      projectId,
      pageId
    });
    return rxjs.firstValueFrom(this.response$.pipe(operators.first(message.isResponseOf(sid)), operators.timeout(constants.CONFIG.responseTimeout)));
  }

  async getActiveSocket() {
    if (this.socket && this.socket.readyState < 2) {
      return this.socket;
    }

    const socket = await this.openNewSocket();
    this.room = null;
    return this.socket = socket;
  }

  async openNewSocket() {
    const socket = new isomorphicWebsocket.IsomorphicWebSocket({
      url: constants.CONFIG.endpoint,
      options: {
        headers: {
          Origin: constants.CONFIG.origin,
          Cookie: util.getAuthCookieValue(this.options.token ?? '')
        }
      },
      serializer: message.scrapboxSerializer,
      deserializer: message.scrapboxDeserializer
    });
    const [, data] = await rxjs.firstValueFrom(socket.message.pipe(operators.first(message.isConnectionMessage)));
    socket.send(constants.packetTypes.connected);
    rxjs.interval(data.pingInterval).pipe(operators.takeUntil(socket.close)).subscribe(() => socket.send(constants.packetTypes.ping));
    return socket;
  }

}

exports.ScrapboxWebsocketHandler = ScrapboxWebsocketHandler;
//# sourceMappingURL=websocket-handler.js.map
