import { Subject, concatMap, firstValueFrom, interval } from 'rxjs';
import { first, timeout, takeUntil } from 'rxjs/operators';
import { CONFIG, packetTypes } from './constants.js';
import { IsomorphicWebSocket } from './internal/isomorphic-websocket.js';
import { toSocketIoMessagePayload, isResponseMessageOf, isResponseOf, scrapboxSerializer, scrapboxDeserializer, isConnectionMessage } from './internal/message.js';
import { createChanges } from './internal/request.js';
import { getAuthCookieValue } from './internal/util.js';

class ScrapboxWebsocketHandler {
    options;
    socket = null;
    room = null;
    sid = 0;
    request$ = new Subject();
    response$ = new Subject();
    constructor(options) {
        this.options = options;
        this.request$
            .pipe(concatMap(async ({ sid, data, projectId, pageId }) => {
            const socket = await this.getActiveSocket();
            socket.send(toSocketIoMessagePayload(sid, data));
            const result = await firstValueFrom(socket.message.pipe(first(isResponseMessageOf(sid)), timeout(CONFIG.responseTimeout)));
            return { sid, result };
        }))
            .subscribe(this.response$);
    }
    async commit({ projectId, userId, pageId, parentId, changes, }) {
        await this.join({ projectId, pageId });
        return this.send({
            projectId,
            pageId,
            data: {
                method: 'commit',
                data: { kind: 'page', userId, projectId, pageId, parentId, changes: createChanges(changes, userId), cursor: null, freeze: true },
            },
        });
    }
    async join({ projectId, pageId }) {
        // join room if needed
        if (this.room?.projectId === projectId && this.room?.pageId === pageId) {
            return;
        }
        await this.send({
            projectId,
            pageId,
            data: {
                method: 'room:join',
                data: { pageId, projectId, projectUpdatesStream: false },
            },
        });
        this.room = { projectId, pageId };
    }
    async send({ data, projectId, pageId }) {
        const sid = String(this.sid++);
        this.request$.next({ sid, data, projectId, pageId });
        return firstValueFrom(this.response$.pipe(first(isResponseOf(sid)), timeout(CONFIG.responseTimeout)));
    }
    async getActiveSocket() {
        if (this.socket && this.socket.readyState < 2) {
            return this.socket;
        }
        const socket = await this.openNewSocket();
        this.room = null;
        return (this.socket = socket);
    }
    async openNewSocket() {
        const socket = new IsomorphicWebSocket({
            url: CONFIG.endpoint,
            options: { headers: { Origin: CONFIG.origin, Cookie: getAuthCookieValue(this.options.token ?? '') } },
            serializer: scrapboxSerializer,
            deserializer: scrapboxDeserializer,
        });
        const [, data] = await firstValueFrom(socket.message.pipe(first(isConnectionMessage)));
        socket.send(packetTypes.connected);
        interval(data.pingInterval)
            .pipe(takeUntil(socket.close))
            .subscribe(() => socket.send(packetTypes.ping));
        return socket;
    }
}

export { ScrapboxWebsocketHandler };
//# sourceMappingURL=websocket-handler.js.map
